@@ -773,7 +773,7 @@ xim_loopback_real_xim_destroy_ic(GXimProtocol *proto,
 		gboolean retval;
 
 		g_xim_messages_warning(G_XIM_PROTOCOL_GET_IFACE (proto)->message,
-				       msg);
+				       "%s", msg);
 		retval = g_xim_connection_cmd_error(G_XIM_CONNECTION (proto),
 						    imid, icid, G_XIM_EMASK_VALID_IMID | G_XIM_EMASK_VALID_ICID,
 						    G_XIM_ERR_BadProtocol,
@@ -803,7 +803,7 @@ xim_loopback_real_xim_set_ic_values(GXimProtocol *proto,
 		gboolean retval;
 
 		g_xim_messages_warning(G_XIM_PROTOCOL_GET_IFACE (proto)->message,
-				       msg);
+				       "%s", msg);
 		retval = g_xim_connection_cmd_error(G_XIM_CONNECTION (proto),
 						    imid, icid, G_XIM_EMASK_VALID_IMID | G_XIM_EMASK_VALID_ICID,
 						    G_XIM_ERR_BadProtocol,
@@ -844,7 +844,7 @@ xim_loopback_real_xim_get_ic_values(GXimProtocol *proto,
 		gboolean retval;
 
 		g_xim_messages_warning(G_XIM_PROTOCOL_GET_IFACE (proto)->message,
-				       msg);
+				       "%s", msg);
 		retval = g_xim_connection_cmd_error(G_XIM_CONNECTION (proto),
 						    imid, icid, G_XIM_EMASK_VALID_IMID | G_XIM_EMASK_VALID_ICID,
 						    G_XIM_ERR_BadProtocol,
