@@ -39,8 +39,8 @@
 #define MAXRESTART 3
 
 typedef struct _IMSettingsProcInfo {
-	GIOChannel     *stdout;
-	GIOChannel     *stderr;
+	GIOChannel     *_stdout;
+	GIOChannel     *_stderr;
 	GPid            pid;
 	GTimeVal        started_time;
 	guint           id;
@@ -52,11 +52,13 @@ struct _IMSettingsProcPrivate {
 	IMSettingsInfo     *info;
 	IMSettingsProcInfo  main;
 	IMSettingsProcInfo  aux;
+	gchar              *desktop;
 	gboolean            shutdown:1;
 };
 enum {
 	PROP_0,
 	PROP_INFO,
+	PROP_DESKTOP,
 	LAST_PROP
 };
 enum {
@@ -109,7 +111,7 @@ _log_write_cb(GIOChannel   *channel,
 	}
 	str = g_string_new(NULL);
 	g_string_append_printf(str, "%s[%lu]: %s", module, (gulong)priv->main.pid, buffer);
-	g_message(str->str);
+	g_message("%s", str->str);
 	g_string_free(str, TRUE);
 	g_free(buffer);
 
@@ -204,7 +206,7 @@ _watch_im_status_cb(GPid     pid,
 							 type_names[type],
 							 module);
 
-			g_critical(message);
+			g_critical("%s", message);
 			g_signal_emit(proc, signals[SIG_NOTIFY_NOTIFICATION], 0, NOTIFY_URGENCY_CRITICAL, title, message, 0);
 			g_free(message);
 			unref = TRUE;
@@ -303,7 +305,7 @@ _start_process(IMSettingsProc     *proc,
 		g_set_error(error, IMSETTINGS_GERROR, IMSETTINGS_GERROR_UNKNOWN,
 			    _("[BUG] %s process is still running [pid: %d]\n"),
 			    type_name, pinfo->pid);
-		g_warning((*error)->message);
+		g_warning("%s", (*error)->message);
 		return FALSE;
 	} else {
 		/* close io channels prior to create new one */
@@ -316,6 +318,7 @@ _start_process(IMSettingsProc     *proc,
 			"GTK_IM_MODULE",
 			"QT_IM_MODULE",
 			"XMODIFIERS",
+			"XDG_CURRENT_DESKTOP",
 			NULL
 		};
 		gsize len, i, j, k;
@@ -353,6 +356,7 @@ _start_process(IMSettingsProc     *proc,
 		envp[j++] = g_strdup_printf("GTK_IM_MODULE=%s", gtkimm);
 		envp[j++] = g_strdup_printf("QT_IM_MODULE=%s", qtimm);
 		envp[j++] = g_strdup_printf("XMODIFIERS=@im=%s", xim);
+		envp[j++] = g_strdup_printf("XDG_CURRENT_DESKTOP=%s", priv->desktop);
 		envp[j++] = NULL;
 
 		cmd = g_strdup_printf("%s %s", prog_name, (prog_args ? prog_args : ""));
@@ -364,12 +368,12 @@ _start_process(IMSettingsProc     *proc,
 					     &ofd, &efd,
 					     error)) {
 			pinfo->pid = pid;
-			pinfo->stdout = g_io_channel_unix_new(ofd);
-			pinfo->stderr = g_io_channel_unix_new(efd);
-			g_io_channel_set_close_on_unref(pinfo->stdout, TRUE);
-			g_io_channel_set_close_on_unref(pinfo->stderr, TRUE);
-			pinfo->oid = g_io_add_watch(pinfo->stdout, G_IO_IN, _log_write_cb, proc);
-			pinfo->eid = g_io_add_watch(pinfo->stderr, G_IO_IN, _log_write_cb, proc);
+			pinfo->_stdout = g_io_channel_unix_new(ofd);
+			pinfo->_stderr = g_io_channel_unix_new(efd);
+			g_io_channel_set_close_on_unref(pinfo->_stdout, TRUE);
+			g_io_channel_set_close_on_unref(pinfo->_stderr, TRUE);
+			pinfo->oid = g_io_add_watch(pinfo->_stdout, G_IO_IN, _log_write_cb, proc);
+			pinfo->eid = g_io_add_watch(pinfo->_stderr, G_IO_IN, _log_write_cb, proc);
 			g_get_current_time(&pinfo->started_time);
 			pinfo->id = g_child_watch_add(pid, _watch_im_status_cb, proc);
 
@@ -482,6 +486,10 @@ imsettings_proc_set_property(GObject      *object,
 	    case PROP_INFO:
 		    priv->info = g_value_dup_object(value);
 		    break;
+	    case PROP_DESKTOP:
+		    g_free(priv->desktop);
+		    priv->desktop = g_strdup(g_value_get_string(value));
+		    break;
 	    default:
 		    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		    break;
@@ -501,6 +509,9 @@ imsettings_proc_get_property(GObject    *object,
 	    case PROP_INFO:
 		    g_value_set_object(value, priv->info);
 		    break;
+	    case PROP_DESKTOP:
+		    g_value_set_string(value, priv->desktop);
+		    break;
 	    default:
 		    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		    break;
@@ -510,13 +521,13 @@ imsettings_proc_get_property(GObject    *object,
 static void
 imsettings_proc_info_finalize(IMSettingsProcInfo *pinfo)
 {
-	if (pinfo->stdout) {
-		g_io_channel_unref(pinfo->stdout);
-		pinfo->stdout = NULL;
+	if (pinfo->_stdout) {
+		g_io_channel_unref(pinfo->_stdout);
+		pinfo->_stdout = NULL;
 	}
-	if (pinfo->stderr) {
-		g_io_channel_unref(pinfo->stderr);
-		pinfo->stderr = NULL;
+	if (pinfo->_stderr) {
+		g_io_channel_unref(pinfo->_stderr);
+		pinfo->_stderr = NULL;
 	}
 	if (pinfo->oid > 0) {
 		g_source_remove(pinfo->oid);
@@ -567,6 +578,12 @@ imsettings_proc_class_init(IMSettingsProcClass *klass)
 							    _("A GObject to be a IMSettingsInfo"),
 							    IMSETTINGS_TYPE_INFO,
 							    G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
+	g_object_class_install_property(object_class, PROP_DESKTOP,
+					g_param_spec_string("desktop",
+							    _("Desktop name"),
+							    _("A desktop name for client"),
+							    NULL,
+							    G_PARAM_READWRITE));
 
 	/* signals */
 
